#
# Authors: Brian Skahan <bskahan@etria.com>
#          Tom von Schwerdtner <tvon@etria.com>
#
# Copyright 2004-2005, Etria, LLP
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

# Standard library imports
from types import StringTypes

# Zope 3 imports
from zope.interface import implements

# Zope 2 imports
from Acquisition import Implicit, aq_parent, aq_inner
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

from OFS.Traversable import Traversable

# CMF imports
from Products.CMFCore.utils import getToolByName

# Product imports
from interfaces import IKeyword


class Keyword(Implicit, Traversable):
    """Implementation of ITopic as a transient wrapper around a simple keyword.
    """

    implements(IKeyword)

    security = ClassSecurityInfo()
    security.declareObjectProtected('View')

    def __init__(self, keywords):
        if isinstance(keywords, StringTypes):
            self.keywords = [keywords,]
        else:
            self.keywords = keywords
        # We will cache catalog queries for performance reasons.
        # This should not cost anything as these objects only last for the
        # duration of a REQUEST anyway.
        self.results = None

    def getPhysicalPath(self):
        """Get the physical path of the object.

        Returns a path (an immutable sequence of strings) that can be used to
        access this object again later, for example in a copy/paste operation.
        getPhysicalRoot() and getPhysicalPath() are designed to operate
        together.
        """
        path = ('keywords',) + tuple(self.keywords)

        p = aq_parent(aq_inner(self))
        if p is not None:
            path = p.getPhysicalPath() + path

        return path

    security.declareProtected('View', 'getId')
    def getId(self):
        """See ITopic.
        """
        return self.getPhysicalPath()[-1]

    security.declareProtected('View', 'getTitle')
    def getTitle(self):
        """See ITopic.
        """
        image = self.getImage()
        if image is not None:
            return image.Title()
        return ''

    security.declareProtected('View', 'title_or_id')
    def title_or_id(self):
        """See ITopic.
        """
        image = self.getImage()
        if image is not None:
            return image.title_or_id()
        return self.getId()

    security.declareProtected('View', 'getDescription')
    def getDescription(self):
        """See ITopic.
        """
        image = self.getImage()
        if image is not None:
            return image.Description()
        return ''

    security.declareProtected('View', 'getImage')
    def getImage(self):
        """See ITopic.
        """
        topic_images = getattr(self.aq_parent.aq_explicit, 'topic_images', None)
        if topic_images is not None:
            hasimage = getattr(topic_images.aq_explicit, self.keywords[0], None)
            if hasimage is not None:
                # Ugh! Acquisition sucks. Can't use the hasimage variable as
                # zope/plone expects an implicit acquisition context, not an
                # explicit one.
                return getattr(topic_images, self.keywords[0])
        # Just to be explicit...
        return None

    security.declareProtected('View', 'getLazyEntries')
    def getLazyEntries(self):
        """See ITopic.
        """
        # Try to return the cached query if possible
        if self.results is not None:
            return self.results
        weblog = self.quills_tool.getParentWeblog(self)
        path = '/'.join(weblog.getPhysicalPath())
        self.results = self.portal_catalog(meta_type='WeblogEntry',
                path={'query':path, 'level': 0},
                Subject = { 'query' : self.keywords,
                            'operator' : 'and'},
                sort_on = 'effective',
                sort_order = 'reverse',
                review_state = 'published')
        return self.results

    def __str__(self):
        """String representation of the Topic"""
        return self.keywords[0]

    def __len__(self):
        """Return the number of posts in this topic.
        """
        # Make sure we've got the results
        self.getLazyEntries()
        return len(self.results)

    def getLastModified(self):
        """See ITopic.
        """
        # Make sure we've got the results for this topic
        self.getLazyEntries()
        if self.results:
            return self.results[0].modified
        else:
            # Just to be explicit about it
            return None

    security.declareProtected('View', 'queryImageRepository')
    def queryImageRepository(self, query=None):
        """Query the image repository"""
        if query is None:
            query = {}
        keywords_query = query.setdefault('Subject',
                                          {'query':[],
                                           'operator':'and'})
        keywords_query['query'].extend(self.keywords)
        print query
        return aq_parent(self).queryImageRepository(query=query)

InitializeClass(Keyword)
